%
%  Wiener Filtering Function for Imaris
%
%  Copyright Torsten Blke, Institute of Anatomy II, University Hospital of Jena, Germany
%
%  Installation:
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%    <CustomTools>
%      <Menu>
%        <Submenu name="Smoothing">
%          <Item name="WienerFiltering" icon="Matlab" tooltip="Applies Wiener Filtering to all selected (visible) channels. Wiener Filtering is appropriate to reduce Gaussian noise.">
%            <Command>MatlabXT::XTWienerFiltering(%i)</Command>
%          </Item>
%        </Submenu>
%      </Menu>
%    </CustomTools> 
%
%  Description:
%   Applies Wiener Filtering to all selected (visible) channels. 
%   Wiener Filtering is appropriate to reduce Gaussian noise.
%   The extension estimates the noise power by itself. 
%   The user can change this estimation in order to adapt the strength of
%   noise suppression.
%   
%   Please note: image data of selected channels will be overwritten by the
%   result!
%
%   This XTension was written for Imaris 8.3.1
%   Necessary Modules: ImarisXT

function XTWienerFiltering(aImarisApplicationID)

% get the application object
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    % called from workspace
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% ask user for kernel size
prompt = {'Enter kernel size X:', 'Enter kernel size Y:'};
dlg_title = 'Wiener Filtering';
num_lines = 1;
def = {'3', '3'};
answer = inputdlg(prompt,dlg_title,num_lines,def);
if isempty(answer), return, end    
nhood(1) = str2num(answer{1});
nhood(2) = str2num(answer{2});

% get dataset clone
try 
    vDataSet = vImarisApplication.GetDataSet.Clone;
catch er                
    msgbox(sprintf(['Please open a dataset before starting Wiener Filtering.\n\n', er.message]));
    return
end

% get list of visible channels
nVisCh = 0;
for vChannel = 1:vDataSet.GetSizeC
    if vImarisApplication.GetChannelVisibility(vChannel-1);
        nVisCh = nVisCh+1;
        iVisCh(nVisCh) = vChannel;
    end    
end 
if nVisCh == 0
    msgbox(sprintf(['Please select at least one channel before starting Wiener Filtering.\n\n']));
    return;    
end

% estimate noise parameter
noise = zeros(1,nVisCh);
def = cell(1,nVisCh);
prompt = cell(1,nVisCh);
for iCh = 1:nVisCh    
    vSlice = floor(vDataSet.GetSizeZ/2);
    vTime = floor(vDataSet.GetSizeT/2);
    data = double(vDataSet.GetDataSliceFloats(vSlice, iVisCh(iCh)-1, vTime));
    % Estimate the local mean of f.
    localMean = filter2(ones(nhood), data) / prod(nhood);
    % Estimate of the local variance of f.
    localVar = filter2(ones(nhood), data.^2) / prod(nhood) - localMean.^2;
    % Estimate the noise power 
    noise(iCh) = mean2(localVar);
    def{iCh} = num2str(noise(iCh), 1);
    prompt{iCh} = strcat('Noise power channel ', num2str(iVisCh(iCh)));   
end
if sum(noise) == 0
    msgbox(sprintf(['Error while estimating noise power.\n\n', ...
            'Please let Imaris load the full dataset before starting Wiener Filtering']));
    return;    
end

% ask for noise parameter to use
answer = inputdlg(prompt,dlg_title,num_lines,def);
for iCh = 1:nVisCh  
    noise(iCh) = str2num(answer{iCh});
end  
if isempty(answer), return, end    

% initialize waitbar
vProgressTotalCount = vDataSet.GetSizeT*vDataSet.GetSizeZ*nVisCh;
vProgressDisplay = waitbar(0, 'Wiener Filtering');
vProgressCount = 0;

for iCh = 1:nVisCh     
    for vTime = 1:vDataSet.GetSizeT
        for vSlice = 1:vDataSet.GetSizeZ
            originalData = double(vDataSet.GetDataSliceFloats(vSlice-1, iVisCh(iCh)-1, vTime-1));
            filteredData = wiener2(originalData, nhood, noise(iCh));

            try
                if strcmp(vDataSet.GetType,'eTypeUInt8')
                    vDataSet.SetDataSliceBytes(uint8(filteredData), vSlice-1,iVisCh(iCh)-1,vTime-1);
                elseif strcmp(vDataSet.GetType,'eTypeUInt16')
                   vDataSet.SetDataSliceShorts(uint16(filteredData), vSlice-1,iVisCh(iCh)-1,vTime-1);
                elseif strcmp(vDataSet.GetType,'eTypeFloat')
                   vDataSet.SetDataSliceFloats(single(filteredData), vSlice-1,iVisCh(iCh)-1,vTime-1);
                end
            catch er
                close(vProgressDisplay);
                msgbox(sprintf(['The result of the expression is not a valid dataset.\n\n', ...
                    'Possible causes: invalid result size.\n\n', er.message]));
                return
            end
            
            vProgressCount = vProgressCount + 1;
            waitbar(vProgressCount/vProgressTotalCount, vProgressDisplay);
        end
    end    
end

vImarisApplication.SetDataSet(vDataSet);
close(vProgressDisplay);
end

